
#include <vector>
#include <match.hh>

#define ISAPNP_CARD_DEVS	8
#include <linux/isapnp.h>

class isapnp_module {
public:
  std::string module;

  uint16_t cardvendor, carddevice;

  struct {
    unsigned short vendor, function;
  } devs[ISAPNP_CARD_DEVS];       /* logical devices */
  int ndevs;

  uint32_t driver_data;
  
  bool operator== (const isapnp_module& other);

  void read_from_sysfs (const std::string& file);
  void read_from_proc (const std::string& file);
  void read_from_env ();
};

std::ostream& operator<< (std::ostream& os, const isapnp_module& module);

class isapnp_subsystem : public basic_subsystem
{
public:
  
  virtual ~isapnp_subsystem  () {}
  
  virtual std::string name () { return "isapnp"; }
  
  virtual void hotplug ();
  virtual void hotplug_all ();
  virtual void list ();
  
private:

  void hotplug (const isapnp_module& m);
  void list (const isapnp_module& m);

  void match_modules (isapnp_module& m,
		      void(isapnp_subsystem::*op)(const isapnp_module&));
  
  void parse_modules (const std::string &fname);
  void parse_modules ();
  void for_each (void(isapnp_subsystem::*op)(const isapnp_module&));
  
  static std::vector<isapnp_module> modules;
};
