
#include <string>
#include <vector>
#include <iostream>

#include <os.hh>

#include <match.hh>

#include <pci-match.hh>
#include <usb-match.hh>
#include <isapnp-match.hh>
#include <scsi-match.hh>
#include <input-match.hh>

#include <firmware-match.hh>

/* We are either called from the kernel for each device hot-plugged
   into the system or at system startup to load the modules for the
   devices already present- */

/* With exactly one argument, not starting with a dash ('-'), it is
   from kernel notifing a hotplug.
   Without any argument we list the available devices, and with
   arguments starting wit a dash, such as "-synth" special functions
   are triggered. */

int main (int argc, char* argv[])
{
  //openlog("HP++", LOG_PID|/* DEBUG!!*/LOG_NDELAY|LOG_PERROR, LOG_KERN);

  std::string mode; // "": hotplug ""
  std::string subsystem;

  /* poor mans argument list pasting - we have a full blown
     template based paserer - hower it if a bit overfeatured here */
  ++argv;
  if (*argv && *argv[0] == '-') {
    mode = *argv++;
    mode.erase (0, 1);
  }
  
  if (*argv)
    subsystem = *argv++;
  
  if (mode.empty() && subsystem.empty())
    mode = "list";
  
  std::cout << "hotplug++: mode: '" << mode
	    << "', subsystem: '" << subsystem << "'" << std::endl;
  
  std::vector<basic_subsystem*> subsystems;
  // fill the subsystems vector
  pci_subsystem pci;
  subsystems.push_back (&pci);
  usb_subsystem usb;
  subsystems.push_back (&usb);
  isapnp_subsystem isapnp;
  subsystems.push_back (&isapnp);
  scsi_subsystem scsi;
  subsystems.push_back (&scsi);
  input_subsystem input;
  subsystems.push_back (&input);

  firmware_subsystem firmware;
  subsystems.push_back (&firmware);
  
  for (std::vector<basic_subsystem*>::iterator it = subsystems.begin();
       it != subsystems.end(); ++it)
    {
      // classic hotplugging
      if (mode.empty())
	{
	  if ((*it)->name() == subsystem)
	    (*it)->hotplug();
	}
      else if (subsystem.empty() || (*it)->name() == subsystem)
	{
	  if (mode == "list")
	    (*it)->list();
	  else if (mode == "synth")
	    (*it)->hotplug_all();
	}
    }

  // if we are not run from udevd, which already handles the other hotplug
  // agents, we might have to run the agents ...
  if (!getenv("UDEVD_EVENT"))
  {
    // only run the agents during kernel calls - not manual invocation
    if (mode.empty())
      run_hotplug_agents (subsystem);
  }

  //closelog();
  return 0;
}
