
#include <vector>
#include <match.hh>

class pci_module {
public:

  std::string module;
  unsigned int vendor, device, subvendor, subdevice, pciclass;
  unsigned int class_mask, driver_data;

  bool operator== (const pci_module& other);

  void read_from_sysfs (const std::string& file);
  void read_from_proc (const std::string& file);
  void read_from_env ();
};

std::ostream& operator<< (std::ostream& os, const pci_module& module);

class pci_subsystem : public basic_subsystem
{
public:
  
  virtual ~pci_subsystem  () {}
  
  virtual std::string name () { return "pci"; }
  
  virtual void hotplug ();
  virtual void hotplug_all ();
  virtual void list ();
  
private:

  void hotplug (const pci_module& m);
  void list (const pci_module& m);

  void match_modules (pci_module& m,
		      void(pci_subsystem::*op)(const pci_module&));
  
  void parse_modules (const std::string &fname);
  void parse_modules ();
  void for_each (void(pci_subsystem::*op)(const pci_module&));
  
  static std::vector<pci_module> modules;
};
